#!/bin/bash
# 
# Bash script for drawing as many pies on desktop as mounted disks
#
# (C) 2011-2012 Felix Hauri - felix@f-hauri.ch
# Licensed under terms of LGPL v3. www.gnu.org

# syntaxe: Simply run script (maybo after setting DESTDIR bellow).

export DESTDIR=$(/bin/ls -1dt $HOME/{Bureau,Desktop} 2>/dev/null | head -n1)
# export DESTDIR=$HOME/Bureau
export SVGIDENT=dfPieSvg

die() {
    echo >&2 $0 - Error: $@
    exit 1
}
[ -d $DESTDIR ] || die "Desktop dir '$DESTDIR' not found"

export TEMPDIR=$(mktemp -d /tmp/${SVGIDENT}-XXXXXX)
[ "$TEMPDIR" ] && [ -d "$TEMPDIR" ] || exit 0
trap "[ -f $TEMPDIR/${SVGIDENT}.svg ] && rm $TEMPDIR/${SVGIDENT}.svg;
      rmdir $TEMPDIR" 0 1 2 3 6 9 15

export OLD_FILES_TO_DELETE="$(
    find $DESTDIR/ -maxdepth 1 \
        \( -size 4 -o -size 5 \) \
        ! -name '*.*' \
        -exec grep -q id=\"${SVGIDENT}- {} \; \
        -printf "%P\n"
    )"

dontDelete() {
    OLD_FILES_TO_DELETE="$(
        echo "$OLD_FILES_TO_DELETE" |\
             sed /^$1$/d
    )"
}

doSvgPie() {
    eval export $(echo val=$1';
        scale=30;
        if (val>=100) val=99.9999;
        x=10+10*c(val/12.5*a(1));
        y=10+10*s(val/12.5*a(1));
        l=0;
        if (val>50) l=1;
        l ; x ; y ; val ' |
        bc -l |
        xargs printf "l=%d x=%.5f y=%.5f v=%.1f")
    cat <<EOF > $TEMPDIR/${SVGIDENT}.svg
<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<svg version="1.0" xmlns="http://www.w3.org/2000/svg"
   xmlns:xlink="http://www.w3.org/1999/xlink"
   xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
   width="${5}px" height="${5}px" viewBox="-1 -1 23 23" id="${SVGIDENT}-$2">
  <defs id="defs4">
    <style type="text/css"><![CDATA[
      text {
         font-style:   bold;
         font-variant: normal;
         font-weight:  normal;
         font-stretch: normal;
         font-size:    4px;
         text-align:   center;
         line-height:  100%;
         writing-mode: lr-tb;
         text-anchor:  middle;
         fill:         #ffffff;
         fill-opacity: .7;
         stroke:       none;
         font-family:  Nimbus Sans L
      }
      text#t1 {
         font-size:    6px;
      }
      circle {
         fill:         #062;
         color:        black;
         fill-opacity: 1;
      }
      path {
         fill:         #800;
         fill-opacity: 1
      }
    ]]></style>
    <filter id="MyFilter">
      <feGaussianBlur in="SourceAlpha" stdDeviation="1" result="blur"/>
      <feOffset in="blur" dx=".5" dy=".7" result="offsetBlur"/>
      <feSpecularLighting in="blur" surfaceScale="5" specularConstant="1" 
         specularExponent="70" lighting-color="#CCC" result="sO1">
        <fePointLight x="-8000" y="-10000" z="23000"/>
      </feSpecularLighting>
      <feComposite in="sO1" in2="SourceAlpha" operator="in" result="sO2"/>
      <feComposite in="SourceGraphic" in2="sO2" operator="arithmetic" 
                  k1="0" k2="1" k3=".9" k4="0" result="litPaint"/>
      <feMerge>
        <feMergeNode in="offsetBlur"/>
        <feMergeNode in="litPaint"/>
      </feMerge>
    </filter>
  </defs>
  <g id="layer1" filter="url(#MyFilter)">
    <circle id="gC" cx="10" cy="10" r="10" />
    <path id="rP" d="M 20,10 A 10,10 0 $l 1 $x,$y L 10,10 z" />
    <text x="10" y="12.2" id="t1"><tspan x="10" y="12.2">$v%</tspan></text>
    <text x="10" y="16.7" id="t2"><tspan x="10" y="16.7">$3</tspan></text>
    <text x="10" y="7" id="t3"><tspan x="10" y="7">$4</tspan></text>
  </g>
</svg>
EOF
}

# dumpsvg $@

export abrev=(k M G T P)

while read prct disk abr size abc sizc iconsize;do
    doSvgPie $prct $disk $size${abrev[$abr]} $sizc${abrev[$abc]} $iconsize
    cmp -s $DESTDIR/$disk $TEMPDIR/${SVGIDENT}.svg ||
        cat $TEMPDIR/${SVGIDENT}.svg >$DESTDIR/$disk
    dontDelete $disk
    done < <(
  df -Pkl |
        awk '/^\//{
              prct=($2-$4)*100/$2;
              unitTot=int(log($2)/log(1024));
              decimTot=1;
              if ((log($2)/log(1024)-unitTot) >0.5) decimTot=0;
              unitCurrent=int(log($4)/log(1024));
              if (unitCurrent == "-inf") unitCurrent=0;
              decimCurrent=1;
              if ((log($4)/log(1024)-unitCurrent) >0.5) decimCurrent=0;
              nameDev=$6
              sub("/","",nameDev);
              gsub("/","-",nameDev);
              if (nameDev=="") nameDev="root";
              iconSize=log($2)^2/3;
              printf "%s %s %d %."decimTot"f %d %."decimCurrent"f %.0f\n",
                 prct, nameDev, unitTot, $2/1024^unitTot, 
                 unitCurrent, $4/1024^unitCurrent, iconSize;
       }' |
    tr a-z A-Z
)
  
[ "$OLD_FILES_TO_DELETE" ] &&
    rm $(printf "$DESTDIR/%s " $OLD_FILES_TO_DELETE)
